// Relay.cpp: implementation of the CRelay class.
// Author : Philip Allen
// Version :1.1
// Date   : 16 March 2006
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Comm.h"
#include "Relay.h"
//#include "USBD096.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

class CComm;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

// Default constructor - sets port to 1
CRelay::CRelay()
{
	m_pCommPort = new CComm();
	m_pCommPort->SetPort("Com1");
}


// Constructor to set port as called
CRelay::CRelay(CString port)
{
	m_pCommPort = new CComm();
	m_pCommPort->SetPort(port);
}


CRelay::~CRelay()
{

}


// Generates the serial commands to open or close a relay.
// To ensure that previous opened or closed relays are not 
// changed the bit states are held as words in 2 arrays.
// Each element of the arrays corresponds to one of the
// control latches. The function uses the iBank value to
// select the appropriate latch and the uPosition to select
// which bit.
// The SendCommand is then called to pass the data to the serial port.

bool CRelay::ActivateRelay(int uPosition, char iBank, int OnOff )
{

    char uRData;
    if ( uPosition <= 8 )
    {
        // Position is under 8 so select word for lower relay bank
        if ( OnOff == 1 )
        {
            // closing relay so change correct bit to 1
			mcLowerRelay[iBank] = mcLowerRelay[iBank] | (1 << (uPosition - 1));
        }
        else
        {
            // opening relay so change correct bit to 0
            mcLowerRelay[iBank] = mcLowerRelay[iBank] & (~ (1 << (uPosition - 1)));
        }
    }    
    else
    {
         // Position is over 8 so subtract 8 and select word for lower relay bank
		uPosition = uPosition - 8;
        if ( OnOff == 1 )
        {
            // closing relay so change correct bit to 1
            mcUpperRelay[iBank] = mcUpperRelay[iBank] | (1 << (uPosition - 1));
        }
        else
        {
           // opening relay so change correct bit to 0
           mcUpperRelay[iBank] = mcUpperRelay[iBank] & (~(1 << (uPosition - 1)));
        }
    }
        
    
	// Change to char
	uRData = mcLowerRelay[iBank];

	SendCommand ("F", uRData);

    uRData = mcUpperRelay[iBank];

    SendCommand("J", uRData);
    return true; 
}


// This function constructs the commands to latch data
// across one of the relay control latches.
// This function relies upon initialisation to set all
// latch outputs low
void CRelay::ActivateBank(char uBank)
{
   char BankWord;
   char uRData;
   
    //Prepare BankWord
    BankWord = 1 << (uBank);
    
    uRData = BankWord | 1;  // bank pos + OE High

    SendCommand ("C", uRData);
    
    //Return bank clock low for next access    
    uRData = 1;  //Clock low OE still high
    SendCommand ("C", uRData);
}


// Used to open all relays 
void CRelay::ClearRelays()
{
    char i;
    char uRData;
    
    // Set Latch enable off
    uRData = 0;
    
    SendCommand ("C", uRData);
    
    // Clear Relay control words
    for( i = 1; i <= 6; i++)
    {
        mcLowerRelay[i] = 0;
        mcUpperRelay[i] = 0;
    }
    
    //set All relay lines low
    uRData = 0;
    SendCommand ("F", uRData);  //All lower port Relays Low
    SendCommand ("J", uRData);  //All upper port Relays Low
    
    //Set all latch ouputs low
    for (i = 1; i<= 6; i++)
    {
        uRData = (1 << i);
        SendCommand ("C", uRData);
    
        //Return bank clock low for next access
        uRData = 0;
        SendCommand ("C", uRData);
    }
    
    // Set Latch enable back to on
    uRData = 1;
    SendCommand ("C", uRData);
    
}


// Closes relay RelNum on bank RelBank
// The last parameter of ActivateRelay set to 1
// denotes close
void CRelay::CloseRelay(int RelNum, int RelBank)
{
    ActivateRelay (RelNum, RelBank, 1);
    ActivateBank (RelBank);
    
}


// Open relay RelNum on bank RelBank
// The last parameter of ActivateRelay set to 0
// denotes open
void CRelay::OpenRelay(int RelNum, int RelBank)
{
    ActivateRelay( RelNum, RelBank, 0);
    ActivateBank (RelBank);
           
}


// Sets the PIC ports to outputs
void CRelay::SetPortStates(CString Dir)
{
   unsigned short uRData;
   
   if ( Dir == "INPUT" )
   {
      uRData = 0xFF;
   }
   else
   {
       uRData = 0;
   }
      
    SendCommand ("B", uRData);  //Set B port
    Sleep(10);

    SendCommand ("E", uRData); //Set C port
    Sleep(10);

    SendCommand ("H", uRData); //Set D port
    Sleep(10);


}


// Called by other external classes to change
// the communcations port as Comm class is 
// only in scope to this class 
BOOL CRelay::SetComPortNum(CString ComPortNum)
{
	mPortNum = ComPortNum;
	return	m_pCommPort->SetPort(ComPortNum);
}


// Wrapper function to pass data to Comm Class
void CRelay::SendCommand(CString command, int cVal)
{
	m_pCommPort->SendCommand(command,(unsigned short)cVal);
}
